from foundationallm.config import Configuration, UserIdentity
from foundationallm.operations import OperationsManager
from foundationallm.plugins import PluginManager
from foundationallm.langchain.agents import (
    LangChainAgentBase,
    LangChainKnowledgeManagementAgent
)

class AgentFactory:
    """
    Factory to determine which agent to use.
    """
    def get_agent(
        self,
        agent_type: str,
        config: Configuration,
        plugin_manager: PluginManager,
        operations_manager: OperationsManager,
        instance_id: str,
        user_identity: UserIdentity) -> LangChainAgentBase:
        """
        Retrieves an agent of the the requested type.

        Parameters
        ----------
        agent_type : str
            The type type assign to the agent returned.
        config : Configuration
            The configuration object containing the details needed for the OrchestrationManager to assemble an agent.
        operations_manager : OperationsManager
            The operations manager object for allowing an agent to interact with the State API.
        instance_id : str
            The unique identifier of the FoundationaLLM instance.
        user_identity : UserIdentity
            The user context under which to execution completion requests.

        Returns
        -------
        AgentBase
            Returns an agent of the requested type.
        """
        if agent_type is None:
            raise ValueError("Agent not constructed. Cannot access an object of 'NoneType'.")
        match agent_type:
            case 'knowledge-management':
                return LangChainKnowledgeManagementAgent(
                        instance_id=instance_id,
                        user_identity=user_identity,
                        config=config,
                        plugin_manager=plugin_manager,
                        operations_manager=operations_manager)
            case _:
                raise ValueError(f'The agent type {agent_type} is not supported.')



from typing import List, Optional, Union, Annotated
from pydantic import BaseModel, Field

from foundationallm.models.orchestration import (
    AnalysisResult,
    ContentArtifact,
    OpenAIImageFileMessageContentItem,
    OpenAITextMessageContentItem
)

class CompletionResponse(BaseModel):
    """
    Response from a language model.
    """
    id: Optional[str] = None
    operation_id: str
    user_prompt: str
    user_prompt_rewrite: Optional[str] = None
    full_prompt: Optional[str] = None
    content: Optional[
        List[
            Annotated[
                Union[
                    OpenAIImageFileMessageContentItem,
                    OpenAITextMessageContentItem
                ],
                Field(discriminator='type')
            ]
        ]
    ] = None
    analysis_results: Optional[List[AnalysisResult]] = []
    content_artifacts: Optional[List[ContentArtifact]] = []
    user_prompt_embedding: Optional[List[float]] = []
    prompt_tokens: int = 0
    completion_tokens: int = 0
    total_tokens: int = 0
    total_cost: float = 0.0
    errors: Optional[List[str]] = []



from typing import List, Optional
from pydantic import BaseModel, Field
from foundationallm.models.attachments import AttachmentProperties
from foundationallm.models.messages import MessageHistoryItem

class CompletionRequestBase(BaseModel):
    """
    Base class for completion requests.
    """
    operation_id: str = Field(description="The operation ID for the completion request.")
    session_id: Optional[str] = Field(None, description="The session ID for the completion request.")
    user_prompt: str = Field(description="The user prompt for the completion request.")
    user_prompt_rewrite: Optional[str] = Field(None, description="The user prompt rewrite for the completion request.")
    message_history: Optional[List[MessageHistoryItem]] = Field(list, description="The message history for the completion.")
    attachments: Optional[List[AttachmentProperties]] = Field(list, description="The attachments collection for the completion request.")



import os
import requests
import aiohttp
from foundationallm.config import Configuration, UserIdentity
from foundationallm.models.authentication import AuthenticationTypes, AuthenticationParametersKeys
from foundationallm.models.resource_providers.configuration import APIEndpointConfiguration

class HttpClientService:
    """
    Class for creating an HTTP client session based on an API endpoint configuration.
    """
    def __init__(self, api_endpoint_configuration: APIEndpointConfiguration, user_identity: UserIdentity, config: Configuration):
        self.config = config
        self.user_identity = user_identity
        self.api_endpoint_configuration = api_endpoint_configuration
        self.base_url = self.api_endpoint_configuration.url.rstrip('/')
        env = os.environ.get('FOUNDATIONALLM_ENV', 'prod')
        self.verify_certs = False if env == 'dev' else True
        self.time_out = self.api_endpoint_configuration.timeout_seconds

        # build headers
        headers = {}
        headers["Content-Type"] = "application/json"
        headers["charset"] = "utf-8"
        if self.api_endpoint_configuration.authentication_type == AuthenticationTypes.API_KEY:
            api_key_value_prefix = self.api_endpoint_configuration.authentication_parameters.get(AuthenticationParametersKeys.API_KEY_PREFIX, "")
            api_key_value = self.config.get_value(self.api_endpoint_configuration.authentication_parameters.get(AuthenticationParametersKeys.API_KEY_CONFIGURATION_NAME))
            api_key_header_name = self.api_endpoint_configuration.authentication_parameters.get(AuthenticationParametersKeys.API_KEY_HEADER_NAME)
            headers[api_key_header_name] = api_key_value_prefix + api_key_value
        else:
            raise Exception(f"Authentication type {self.api_endpoint_configuration.authentication_type} is not currently supported.")

        # Check for base URL exceptions.
        for url_exception in self.api_endpoint_configuration.url_exceptions:
            if url_exception.user_principal_name == self.user_identity.upn and url_exception.enabled:
                self.base_url = url_exception.url.rstrip('/')
                break

        self.headers = headers

    def get(self, endpoint: str):
        """
        Execute a synchronous GET request.
        """
        with requests.Session() as session:
            session.headers.update(self.headers)
            url = self.base_url + endpoint
            response = session.get(url, timeout=self.time_out, verify=self.verify_certs)
            response.raise_for_status()
            return response.json()

    def post(self, endpoint: str, data: dict = None):
        """
        Execute a synchronous POST request.
        """        
        with requests.Session() as session:
            session.headers.update(self.headers)
            url = self.base_url + endpoint            
            response = session.post(url, data=data, timeout=self.time_out, verify=self.verify_certs)            
            response.raise_for_status()
            return response.json()

    async def get_async(self, endpoint: str):
        """
        Execute an asynchronous GET request.
        """
        async with aiohttp.ClientSession(headers=self.headers) as session:
            url = self.base_url + endpoint
            async with session.get(url, timeout=self.time_out, ssl=self.verify_certs) as response:
                response.raise_for_status()
                return await response.json()

    async def post_async(self, endpoint: str, data: dict = None):
        """
        Execute an asynchronous POST request.
        """
        async with aiohttp.ClientSession(headers=self.headers) as session:
            url = self.base_url + endpoint
            async with session.post(url, data=data, timeout=self.time_out, ssl=self.verify_certs) as response:
                response.raise_for_status()
                return await response.json()
